library(survival)
library(parallel)
source('./func.R')

scene <- 1  # scenario number in the paper
Simu <- 500 # number of simulation replications
n <- 200    # sample size
tau <- 35

RNGkind("L'Ecuyer-CMRG") 
set.seed(Simu)            
ptrt <- 0.5 # treatment assignment probability

########################################
# values of RSBs under null hypothesis #
########################################

test.del0.RMST  <- 0
test.del0.ratio <- 0.497
test.del0.dif   <- 0.448
test.del0.PP    <- 0

########################################
# true values of RSBs under scenario 1 #
########################################

del0.RMST  <- 0
del0.ratio <- 0.497
del0.dif   <- 0.448
del0.PP    <- 0

results <- NULL # record the results
for (sim in 1:Simu){
err <- TRUE
while (err){
tryCatch({
# Data generation #
T1 <- sim.T(n, lambda = 0.295, k = 2)
T0 <- sim.T(n, lambda = 0.295, k = 2)
X <- rbinom(n, size = 1, prob = 0.5)
A <- rbinom(n, size = 1, prob = ptrt)
T <- A*T1+(1-A)*T0
C <- runif(n, max = 75)
d <- (T < C)
Y <- pmin(T,C)
d[(d == 0 & Y >= tau)] <- 1
Y <- pmin(Y,tau)
dat <- data.frame(cbind(A,X,Y,d))

S1 <- c(1:n)[(A==1)]
S0 <- c(1:n)[(A==0)]
S11 <- c(1:n)[(A==1 & d==1)]
S01 <- c(1:n)[(A==0 & d==1)]
n1 <- sum(A)
n0 <- n - n1
n11 <- sum(A==1 & d==1)
n01 <- sum(A==0 & d==1)
Y1 <- Y[S1]
Y0 <- Y[S0]
Y1.cc <- Y[S11]
Y0.cc <- Y[S01]
d1 <- d[S1]
d0 <- d[S0]
X1 <- X[S1]
X0 <- X[S0]

p1 <- cal_KM(Y1,1-d1,Y1)
p0 <- cal_KM(Y0,1-d0,Y0)
if (any(p1 <= 1E-5) | any(p0 <= 1E-5)) stop()
p1.cc <- p1[match(S11,S1)]
p0.cc <- p0[match(S01,S0)]

######################################
# curde estimator ignoring censoring #
######################################

del.cc.RMST  <- mean(apply(matrix(Y1.cc,1,),2,"-",Y0.cc)) 
del.cc.ratio <- mean(apply(matrix(Y1.cc,1,),2,">",Y0.cc * 1.1)) 
del.cc.dif   <- mean(apply(matrix(Y1.cc,1,),2,">",Y0.cc))
del.cc.PP    <- mean(apply(matrix(Y1.cc,1,),2,">",Y0.cc))-mean(apply(matrix(Y0.cc,1,),2,">",Y1.cc)) 

###############################
# Kaplan-Meier IPCW estimator #
###############################

del.im.RMST  <- sum(apply(matrix(Y1.cc,1,),2,"-",Y0.cc)/apply(matrix(p1.cc,1,),2,"*",p0.cc))/(n1*n0)
del.im.ratio <- sum(apply(matrix(Y1.cc,1,),2,">",Y0.cc * 1.1)/apply(matrix(p1.cc,1,),2,"*",p0.cc))/(n1*n0)
del.im.dif   <- sum(apply(matrix(Y1.cc,1,),2,">",Y0.cc)/apply(matrix(p1.cc,1,),2,"*",p0.cc))/(n1*n0)
del.im.PP    <- sum((apply(matrix(Y1.cc,1,),2,">",Y0.cc)-t(apply(matrix(Y0.cc,1,),2,">",Y1.cc)))/apply(matrix(p1.cc,1,),2,"*",p0.cc))/(n1*n0)

Boots <- Boot(odata = dat, delhat = c(del.im.RMST,del.im.ratio,del.im.dif,del.im.PP)) # Bootstrap

#################################################################################################
# 1. Wald-type test based on the asymptotic normality of the curde estimator ignoring censoring #
# 2. Wald-type test based on the asymptotic normality of the Kaplan-Meier IPCW estimator        #
# 3. Empirical likelihood ratio test based on the Kaplan-Meier IPCW estimator                   #
# for different restricted survival benefits (RSBs)                                             #
#################################################################################################

# Restricted mean survival: E(min(T1,\tau) - min(T0,\tau)) #
J.vec <- Jack.RMST(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * del0.RMST
ELRtest <- cal.weight.JEL(matrix(g.vec,,1))$ELR

p.CC.RMST <- (abs(del.cc.RMST-test.del0.RMST)/Boots$se.cc.RMST>=qnorm(0.975))
CI.AN.L <- del.cc.RMST-qnorm(0.975)*Boots$se.cc.RMST
CI.AN.U <- del.cc.RMST+qnorm(0.975)*Boots$se.cc.RMST
cover.cc.RMST <- (CI.AN.L<del0.RMST & CI.AN.U>del0.RMST)

p.IM.RMST <- (abs(del.im.RMST-test.del0.RMST)/Boots$se.im.RMST>=qnorm(0.975))
CI.AN.L <- del.im.RMST-qnorm(0.975)*Boots$se.im.RMST
CI.AN.U <- del.im.RMST+qnorm(0.975)*Boots$se.im.RMST
cover.im.RMST <- (CI.AN.L<del0.RMST & CI.AN.U>del0.RMST)
quan <- Boots$avg.RMST * qchisq(0.95, df = 1)
p.ELR.RMST <- (cal.weight.JEL(matrix(J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * test.del0.RMST,,1))$ELR>=quan)
CI.RMST <- findUL.RMST(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, MLE = del.im.RMST, level = quan, del0 = del0.RMST)

# Restricted change of longer survival: P(min(T1,\tau) > m * min(T0,\tau)) #
J.vec <- Jack.ratio(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, m = 1.1)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * del0.ratio
ELRtest <- cal.weight.JEL(matrix(g.vec,,1))$ELR

p.CC.ratio <- (abs(del.cc.ratio-test.del0.ratio)/Boots$se.cc.ratio>=qnorm(0.975))
CI.AN.L <- del.cc.ratio-qnorm(0.975)*Boots$se.cc.ratio
CI.AN.U <- del.cc.ratio+qnorm(0.975)*Boots$se.cc.ratio
cover.cc.ratio <- (CI.AN.L<del0.ratio & CI.AN.U>del0.ratio)

p.IM.ratio <- (abs(del.im.ratio-test.del0.ratio)/Boots$se.im.ratio>=qnorm(0.975))
CI.AN.L <- del.im.ratio-qnorm(0.975)*Boots$se.im.ratio
CI.AN.U <- del.im.ratio+qnorm(0.975)*Boots$se.im.ratio
cover.im.ratio <- (CI.AN.L<del0.ratio & CI.AN.U>del0.ratio)
quan <- Boots$avg.ratio * qchisq(0.95, df = 1)
p.ELR.ratio <- (cal.weight.JEL(matrix(J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * test.del0.ratio,,1))$ELR>=quan)
CI.ratio <- findUL.ratio(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, MLE = del.im.ratio, level = quan, del0 = del0.ratio)

# Restricted change of longer survival: P(min(T1,\tau) > m + min(T0,\tau)) #
J.vec <- Jack.dif(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, m = 0)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * del0.dif
ELRtest <- cal.weight.JEL(matrix(g.vec,,1))$ELR

p.CC.dif <- (abs(del.cc.dif-test.del0.dif)/Boots$se.cc.dif>=qnorm(0.975))
CI.AN.L <- del.cc.dif-qnorm(0.975)*Boots$se.cc.dif
CI.AN.U <- del.cc.dif+qnorm(0.975)*Boots$se.cc.dif
cover.cc.dif <- (CI.AN.L<del0.dif & CI.AN.U>del0.dif)

p.IM.dif <- (abs(del.im.dif-test.del0.dif)/Boots$se.im.dif>=qnorm(0.975))
CI.AN.L <- del.im.dif-qnorm(0.975)*Boots$se.im.dif
CI.AN.U <- del.im.dif+qnorm(0.975)*Boots$se.im.dif
cover.im.dif <- (CI.AN.L<del0.dif & CI.AN.U>del0.dif)
quan <- Boots$avg.dif * qchisq(0.95, df = 1)
p.ELR.dif <- (cal.weight.JEL(matrix(J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * test.del0.dif,,1))$ELR>=quan)
CI.dif <- findUL.dif(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, MLE = del.im.dif, level = quan, del0 = del0.dif)

# Restricted net change of longer survival: 
# P(min(T1,\tau) > m + min(T0,\tau)) - P(min(T0,\tau) > m + min(T1,\tau))
J.vec <- Jack.PP(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, m = 0)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * del0.PP
ELRtest <- cal.weight.JEL(matrix(g.vec,,1))$ELR

p.CC.PP <- (abs(del.cc.PP-test.del0.PP)/Boots$se.cc.PP>=qnorm(0.975))
CI.AN.L <- del.cc.PP-qnorm(0.975)*Boots$se.cc.PP
CI.AN.U <- del.cc.PP+qnorm(0.975)*Boots$se.cc.PP
cover.cc.PP <- (CI.AN.L<del0.PP & CI.AN.U>del0.PP)
p.IM.PP <- (abs(del.im.PP-test.del0.PP)/Boots$se.im.PP>=qnorm(0.975))
CI.AN.L <- del.im.PP-qnorm(0.975)*Boots$se.im.PP
CI.AN.U <- del.im.PP+qnorm(0.975)*Boots$se.im.PP
cover.im.PP <- (CI.AN.L<del0.PP & CI.AN.U>del0.PP)
quan <- Boots$avg.PP * qchisq(0.95, df = 1)
p.ELR.PP <- (cal.weight.JEL(matrix(J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * test.del0.PP,,1))$ELR>=quan)
CI.PP <- findUL.PP(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, MLE = del.im.PP, level = quan, del0 = del0.PP)

# Final results #
res <- c(sim, 
del.cc.RMST , del.im.RMST , p.CC.RMST , p.IM.RMST , p.ELR.RMST , cover.cc.RMST , cover.im.RMST , CI.RMST ,
del.cc.ratio, del.im.ratio, p.CC.ratio, p.IM.ratio, p.ELR.ratio, cover.cc.ratio, cover.im.ratio, CI.ratio,
del.cc.dif  , del.im.dif  , p.CC.dif  , p.IM.dif  , p.ELR.dif  , cover.cc.dif  , cover.im.dif  , CI.dif  ,
del.cc.PP   , del.im.PP   , p.CC.PP   , p.IM.PP   , p.ELR.PP   , cover.cc.PP   , cover.im.PP   , CI.PP)
results <- rbind(results, res)
err <- FALSE
},error = function(e){err <- TRUE})
}
}
